{{

This is the base code for the weather station based on the Parallax Propeller chip.

We used a transaction engine model.

V2 

}}

OBJ
  pst  :"Parallax Serial Terminal"
  num   : "Numbers"
CON                                                                   
  _CLKMODE=XTAL1+ PLL16X   'The system clock spec
  _XINFREQ      = 5_000_000
  MAXREC = 35
  BUFFMAX = 5
  DEBUG = 1
  MAXDESC = 10
  NUMBEROFENTRIES = 0
  OWMAX = 10
   
{{
This is the record definition that will be passed program to program. There is a head and a body
with the header being the first 14 bytes and the data being the next.
There can be from 1 to 6 variables, all numeric, all single integer. for compatability, I have
ordered them Most Significant Byte (MSB) first.
 
Byte 0 is the Target of the transaction. W = Weather, H=Home, F=Fountain etc
Byte 1 is the type. A= Alarm, R = Request, D=Databack, I=Inform
Byte 2 is the transaction ID. an integer from 0 to 255. This tells what kind of Data is following
Byte 3 is a delimiter so we can feed it to a PC and have it parsed
Byte 4 is the Address (XBEE) that sent it to us. We do this because we could have more than one of a station
Byte 5 is the delimiter
Byte 6 is the Address that we send it to
Byte 7 is a Delimiter
Byte 8,9,10,11,12,13 are a "datestamp" in the form YMDHMS
Byte 14 is a delimiter
Byte 15 is the number of variables that follow
Byte 16 is a delimiter

That ends the header. Now each of the variables follow in 2 byte groups. MSB first.
Bytes 17-18    inVar0
Bytes 20-21    inVar1
Bytes 23-24    inVar2
Bytes 26-27    inVar3
Bytes 29-30    inVar4
Bytes 32-33    inVar5

In between each is a Comma for PC compatability.

The string ends with a comma in the last position. We send all 6 variables, even if they are all
zeros 
}}
  TRAN  = 0
  TYPE  = 1
  TID   = 2
  SID   = 4
  RID   = 6
  IYR   = 8
  IMT   = 9
  IDY   = 10
  IHR   = 11
  IMN   = 12
  ISC   = 13
  VCT   = 15
  iVAR  = 17

{{
This is the set of variables for the engine. Comment out the sections you don't want but
only ONE can be active.
}}
' WeatherStation
  WINDSPEED = 1
  AIRTEMP = 2
  WINDDIR = 3
  BARO  = 4
  HUMIDITY = 5
  LIGHTNING = 6
  SOLAR = 7
  RAINFALL = 8
  WATERTEMP = 9
  WATERSPEED = 10
  WATERCLEAR = 11
  WATERDIR   = 12
  TIDELEVEL = 13
  SALINITY = 14
  LIGHTLEVEL = 15
  VARCNT = 15
{{
' Aquarium
  WATERLEVEL = 1
  PH = 2
  LIGHTS = 3
  CLARITY = 4
  VARCNT = 4
}}
{{
' Fountain
  WATERLEVEL = 1
  PUMPSTATE = 2
  VARCNT = 2   

}}
{{
' Boat
  BATTERYV1 = 1
  BATTERYV2 = 2
  BATTERYV3 = 3
  BATTERYV4 = 4
  BATTERYA1 = 5
  BATTERYA2 = 6
  BATTERYA3 = 7
  BATTERYA4 = 8
  MOTORSPEED = 9
  WATERSPEED = 10
  WATERTEMP = 11
  WATERCLAR = 12
  SALINITY = 13
  VARCNT = 13    
}}
{{
' Groundskeeper
  IRRIGATION = 0
  PUMPSTATE = 1
  GROUNDMOIST = 2
  LEAFMOIST = 3
  VARCNT = 3
}}  
{{
' Tower Station
  WINDSPEED = 1
  AIRTEMP = 2
  WINDDIR = 3
  LIGHTNING = 5
  SOLAR = 6
  RAINFALL = 7
  WATERTEMP = 8
  WATERSPEED = 9
  WATERCLEAR = 10
  WATERDIR   = 11
  TIDELEVEL = 12
  SALINITY = 13
  LIGHTLEVEL = 14
  VARCNT = 14
   
}}  
  
VAR
  byte PrseChar, inChar, otChar
  Byte inDataBuffer[MAXREC * BUFFMAX]
  byte inString[MAXREC]
  Byte OtDataBuffer[MAXREC * BUFFMAX] 
  byte OutRecord[MAXREC], InRecord[MaxRec]
  Byte frmAddr, toAddr, myAddr
  Word DataVal[20], DataValAvg[20], DataValSamp[20], DataValDir[20]
  
  LONG OWData[2 * OWMAX ]
  Byte DataDesc[NUMBEROFENTRIES * MAXDESC]
'Global Record definitions 
  Byte inTran, InType, inTID,InSID, InRID, InYr, InMth,InDay,InHr, InMin, InSec
  Long inVar[6]
  
        
  
Pub Main

PrseChar := ","
pst.start(115_200)
waitcnt(clkfreq * 2 + cnt)

DataVal[NUMBEROFENTRIES] := VARCNT

BuildTestRecord
ParseRecord(0)
DebugRecord
ParseRecord(1)
DebugRecord

Pub BuildTestRecord | ptr, StrLen, Idx, VarCount 
  DataVal[WindSpeed] := 134
  DataVal[WindDir] := 270
  FrmAddr := 01
  toAddr := 01
  VarCount := 1    ' This is actually 2. It's plus 1
  ptr := 0
  OutRecord[TRAN] := $41
  OutRecord[TYPE] := $42
  OutRecord[TID] := 1
  outrecord[3] := Prsechar
  OutRecord[SID] := FrmAddr
  outrecord[5] := Prsechar    
  outrecord[RID] := ToAddr
  OutRecord[7] := PrseChar
  OutRecord[IYR] := 10
  OutRecord[IMT] := 08
  OutRecord[IDY] := 22
  OutRecord[IHR] := 12
  OutRecord[IMN] := 06
  OutRecord[ISC] := 30
  OutRecord[14] := PrseChar
  OutRecord[VCT] := VarCount
  OutRecord[16] := PrseChar 
  OutRecord[17] := WindSpeed >>8
  OutRecord[18] := WindSpeed
  OutRecord[19] := PrseChar
  OutRecord[20] := WindDir >>8
  OutRecord[21] := WindDir
  OutRecord[22] := PrseChar
  OutRecord[23] := 0
  ByteMove(@InDataBuffer[Ptr],@OutRecord[0],33)
  DataVal[WaterSpeed] := 4
  DataVal[WaterTemp] := 86
  FrmAddr := 01
  toAddr := 01
  VarCount := 1    ' This is actually 2. It's plus 1
  ptr := 0
  OutRecord[TRAN] := $41
  OutRecord[TYPE] := $42
  OutRecord[TID] := 02
  outrecord[3] := Prsechar
  OutRecord[SID] := FrmAddr
  outrecord[5] := Prsechar    
  outrecord[RID] := ToAddr
  OutRecord[7] := PrseChar
  OutRecord[IYR] := 10
  OutRecord[IMT] := 08
  OutRecord[IDY] := 22
  OutRecord[IHR] := 12
  OutRecord[IMN] := 06
  OutRecord[ISC] := 31
  OutRecord[14] := PrseChar
  OutRecord[VCT] := VarCount
  OutRecord[16] := PrseChar 
  OutRecord[17] := WaterSpeed >>8
  OutRecord[18] := WaterSpeed
  OutRecord[19] := PrseChar
  OutRecord[20] := WaterTemp >>8
  OutRecord[21] := WaterTemp
  OutRecord[22] := PrseChar
  OutRecord[23] := 0
  ByteMove(@InDataBuffer[1 * MAXREC],@OutRecord[0],33)
  pst.str(String("Finish Build"))
  if DEBUG == 1
    waitcnt(clkfreq * 4 + cnt)

Pub ParseRecord(StartPos) |  BuffPos, idx, jdx , VarCount
  pst.Str(String(13,"Starting Parse now",13))
  BuffPos := StartPos * MAXREC
  pst.dec(buffpos)
  InTran := InDataBuffer[TRAN + BuffPos]
  InType := InDataBuffer[TYPE + BuffPos]
  InTID := InDataBuffer[TID + BuffPos]
  inSID := InDataBuffer[SID + BuffPos]
  InRID := InDataBuffer[RID + BuffPos]
  inYr := InDataBuffer[IYR + BuffPos]
  inMth := InDataBuffer[IMT + BuffPos]
  InDay := InDataBuffer[IDY + BuffPos]
  InHr  := inDatabuffer[IHR + BuffPos]
  inMin := inDataBuffer[IMN + BuffPos]
  inSec := inDataBuffer[ISC + BuffPos]
  VarCount := InDataBuffer[VCT + BuffPos]
  repeat idx from 0 to VarCount 
        jdx := IVAR + (idx * 3) + BuffPos
        InVar[idx] := (InDataBuffer[JDX] << 8) + InDataBuffer[JDX+1]
  pst.str(String(13,"Record Complete"))   
  if DEBUG == 1
    waitcnt(clkfreq * 4 + cnt) 

pub debugrecord | idx, jdx 
    pst.str(String(13,"Show the values",13))
    if DEBUG == 1
      waitcnt(clkfreq * 4 + cnt) 
    pst.positionx(0)
    pst.str(String("TRAN"))
    pst.positionx(10)
    pst.str(String("SI"))
    pst.positionx(13)
    pst.str(string("RI"))
    pst.positionX(20)
    pst.str(String("YYMDDHHMMSS"))    
    repeat idx from 0 to 8
          pst.positionx(35 + (4*idx))
          pst.char("V")
          pst.dec(idx)
    pst.str(string(13))
    if DEBUG == 1
      waitcnt(clkfreq * 4 + cnt)         
    pst.positionx(0)
    pst.Char(InTran)
    pst.Char(InType)
    pst.dec(InTID)
    pst.positionx(10)
    pst.DEC(InSID)
    pst.positionx(13)
    pst.Dec(inRID)
    pst.positionX(20)
    pst.Dec(InYr)
    pst.Dec(inMth)
    pst.dec(inDay)
    pst.dec(inHr)
    pst.dec(inMin)
    pst.dec(inSec)
    repeat idx from 0 to 8
        pst.positionX(35 + (idx * 4))
        pst.dec(inVar[idx])        

Pub GetDesc(index) : ptrString
  ptrString := (Index - 1) * MAXDESC

Pub PutDesc(Index,DescVal) | StringLength, idx
  StringLength := strsize(DescVal)
  if StringLength > (MAXDESC - 1)
    StringLength := MAXDESC - 1
    
  repeat idx from 0 to Stringlength
      DataDesc[idx] := byte [DescVal][Idx]

  DataDesc[StringLength+1]~
